/*!
 * @file recorder.h
 * 
 */

#ifndef _RECORDER_H
#define _RECORDER_H

#define STREAM_DURATION   40.0
#define STREAM_FRAME_RATE 20
#define STREAM_NB_FRAMES  ((int)(STREAM_DURATION * STREAM_FRAME_RATE))
#define STREAM_PIX_FMT PIX_FMT_YUV420P


#include <SDL.h>

#ifdef HAVE_AVFORMAT_H
  #include <avformat.h>
#else
  #include <ffmpeg/avformat.h>
#endif

#include "glincl.h"
#include "world_entity.h"

class Recorder : public WorldEntity
{
  private:
    AVFrame* picture;
    uint8_t* buffer;
    int frame_count;
    int buffer_size;

    AVOutputFormat* output_format;
    AVFormatContext* format_context;
    AVCodecContext* codec_context;
    AVCodec *codec;
    AVStream* video_stream;
    AVPacket packet;
    double video_pts;
    
    uint8_t *picture_buf;
    int size;

    int height;
    int width;
    float time;

  public:
    Recorder (const TiXmlElement* root = NULL);
    ~Recorder ();

    virtual void draw () const;
    virtual void tick (float dt);

    virtual void loadParams(const TiXmlElement* root);

  private:
    void initVideo(const char* filename);
    void addVideoStream();
    void openVideo();
    void closeVideo();
    void allocPicture();
    void writeVideoFrame();
    void fillYuvImage();
};

#endif /* _RECORDER_H */
